#!/usr/bin/perl

package GDPlane;

use strict;
use warnings;
use vars qw (@ISA);
use GD;
use Data::Dumper;

# variables
my $xoffs = 25;
my $yoffs = 7;

# create new plane telling how many steps you want on x and y axes
# (default 10) and the step size (default .5)
sub new {
	my ($proto, %args) = @_;
        my $class = ref($proto)||$proto;
	
	my $self = {};
	$self->{img}  = $args{img};
	$self->{hstp} = $args{hstp} || .5;
	$self->{vstp} = $args{vstp} || .5;
	$self->{numx} = $args{numx} || 10;
	$self->{numy} = $args{numy} || 10;
	$self->{linecol} = $args{linecol} || $self->{img}->colorAllocate( 0, 0, 0 );
	$self->{fontcol} = $args{fontcol} || $self->{img}->colorAllocate( 0, 0, 0 );
	$self->{pointcol} = $args{pointcol} || $self->{img}->colorAllocate( 0, 0, 255 );
	$self->{centcol} = $args{centcol} || $self->{img}->colorAllocate( 255, 0, 0 );
	bless ($self, $class);
        return $self;
}

# dump plane data
sub dump{
	my $self = shift;
	print Dumper ($self);	
}

# draw plane giving start x and y coordinates, width and height
sub draw{
	my $self = shift;
	my ($xstart, $ystart, $width, $height) = @_;
	my $i;
	my $numx = $self->{numx};
	my $numy = $self->{numy};
	my $vstp = $self->{vstp};
	my $hstp = $self->{hstp};
	my $linecol = $self->{linecol};
	my $fontcol = $self->{fontcol};

	my $hh = $width/$numx;
	my $vv = $height/$numy;
	
	$self->{xstart} = $xstart;
	$self->{ystart} = $ystart;
	$self->{width} = $width;
	$self->{height} = $height;

	$self->{img}->setStyle($linecol, $linecol, $linecol, gdTransparent, gdTransparent);

	# draw horizontal lines	and numbers on the left of y axis
	for ($i=0;$i<=$numy;$i++){
		my $style = ($i==0 ? $linecol: gdStyled);
		$self->{img}->line($xstart+$xoffs, $ystart-$vv*$i, $xstart+$xoffs+$width, $ystart-$vv*$i, $style);
		
		if ($i!=0){
			$self->{img}->string(gdSmallFont, $xstart, $ystart-$yoffs-$vv*$i, $vstp*$i, $fontcol);
		}
	}

	# draw vertical lines and numbers below x axis
	for ($i=0;$i<=$numx;$i++){
		my $style = ($i==0 ? $linecol: gdStyled);
		$self->{img}->line($xstart+$xoffs+$hh*$i, $ystart, $xstart+$xoffs+$hh*$i, $ystart-$height, $style);
		
		if ($i!=0){
			$self->{img}->string(gdSmallFont, $xstart+$xoffs-2+$hh*$i, $ystart+$yoffs-2, $hstp*$i, $fontcol);
		}
	}
}

sub addPoint{
	my $self = shift;
	my ($x, $y) = @_;
	my $pntsize = 8;

	my $realx = $xoffs+$self->{xstart}+$self->{width}/$self->{numx}*$x/$self->{hstp}; 
	my $realy =        $self->{ystart}-$self->{height}/$self->{numy}*$y/$self->{vstp}; 

	my $polygon = new GD::Polygon();
	$polygon->addPt( $realx-$pntsize, $realy );
	$polygon->addPt( $realx , $realy-$pntsize );
	$polygon->addPt( $realx+$pntsize, $realy );
	$polygon->addPt( $realx, $realy+$pntsize );
	$self->{img}->filledPolygon( $polygon, $self->{pointcol});
}

sub addCentroid{
	my $self = shift;
	my ($x, $y) = @_;
	my $centsize = 10;

	my $realx = $xoffs+$self->{xstart}+$self->{width}/$self->{numx}*$x/$self->{hstp}; 
	my $realy =        $self->{ystart}-$self->{height}/$self->{numy}*$y/$self->{vstp}; 

	for (my $i=1; $i<$centsize; $i++){
		$self->{img}->arc($realx, $realy, $i, $i, 0, 360, $self->{centcol});
	}

}



1;
